/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __NNFW_SUPPORT_TFLITE_TENSOR_VIEW_H__
#define __NNFW_SUPPORT_TFLITE_TENSOR_VIEW_H__

#include "tensorflow/contrib/lite/interpreter.h"

#include "util/tensor/Shape.h"
#include "util/tensor/Index.h"
#include "util/tensor/Reader.h"
#include "util/tensor/NonIncreasingStride.h"

namespace nnfw
{
namespace support
{
namespace tflite
{

template<typename T> class TensorView;

template<> class TensorView<float> final : public nnfw::util::tensor::Reader<float>
{
public:
  TensorView(const nnfw::util::tensor::Shape &shape, float *base);

public:
  const nnfw::util::tensor::Shape &shape(void) const { return _shape; }

public:
  float at(const nnfw::util::tensor::Index &index) const override;
  float &at(const nnfw::util::tensor::Index &index);

private:
  nnfw::util::tensor::Shape _shape;

public:
  float *_base;
  nnfw::util::tensor::NonIncreasingStride _stride;

public:
  // TODO Introduce Operand ID class
  static TensorView<float> make(::tflite::Interpreter &interp, int operand_id);
};

} // namespace tflite
} // namespace support
} // namespace nnfw

#endif // __NNFW_SUPPORT_TFLITE_TENSOR_VIEW_H__
