/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __NNFW_KERNEL_ACL_RESHAPE_COMMON_H__
#define __NNFW_KERNEL_ACL_RESHAPE_COMMON_H__
#include <OperationsUtils.h>
#include <arm_compute/core/TensorShape.h>
#include <arm_compute/core/TensorInfo.h>

// TODO: fix include path in CMakeFiles
#include "IO_accessor.h"
#include "shape.h"

namespace nnfw {
namespace kernel {
namespace acl {

namespace common {

typedef std::function<void (void)> sync_scheduler_f;

template<class TensorT, class LayerT>
bool reshapeGeneric(const void* inputData, const nnfw::rt::Shape& inputShape,
                    void* outputData, const nnfw::rt::Shape& outputShape,
                    sync_scheduler_f sync_scheduler) {

  auto input_shape = util::fromNNShape(inputShape);
  auto output_shape = util::fromNNShape(outputShape);

  TensorT input(arm_compute::TensorInfo(input_shape, arm_compute::Format::F32));
  TensorT output(arm_compute::TensorInfo(output_shape, arm_compute::Format::F32));

  LayerT l;

  l.configure(input.ptr(), output.ptr());

  input.allocate();
  output.allocate();

  TensorAccess<InputAccessor>(input.ref(), (float*)inputData, inputShape);

  l.run();

  sync_scheduler();

  TensorAccess<OutputAccessor>(output.ref(), (float*)outputData, outputShape);

  return true;
}

} // namespace common

} // namespace acl
} // namespace kernel
} // namespace nnfw

#endif // __NNFW_KERNEL_ACL_RESHAPE_COMMON_H__
