/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <OperationsUtils.h>
#include <arm_compute/core/TensorShape.h>
#include <arm_compute/core/TensorInfo.h>

#include <cassert>

// TODO: fix include path in CMakeFiles
#include "../IO_accessor.h"
#include "../shape.h"
#include "../CLUniqueTensor.h"
#include "../DepthwiseConv2D.h"

namespace nnfw {
namespace kernel {
namespace acl {

static void sync_scheduler() {
  arm_compute::CLScheduler::get().sync();
}

bool depthwiseConvFloat32(const float* inputData, const nnfw::rt::Shape& inputShape,
                          const float* filterData, const nnfw::rt::Shape& filterShape,
                          const float* biasData, const nnfw::rt::Shape& biasShape,
                          int32_t padding_left, int32_t padding_right,
                          int32_t padding_top, int32_t padding_bottom,
                          int32_t stride_width, int32_t stride_height,
                          int32_t depth_multiplier, int32_t activation,
                          float* outputData, const nnfw::rt::Shape& outputShape) {
  return common::depthwiseConvFloat32<CLUniqueTensor, arm_compute::CLDepthwiseConvolutionLayer,
  arm_compute::CLActivationLayer>(inputData, inputShape,
      filterData, filterShape,
      biasData, biasShape,
      padding_left, padding_right,
      padding_top, padding_bottom,
      stride_width, stride_height,
      depth_multiplier, activation,
      outputData, outputShape,
      sync_scheduler);
}

} // namespace acl
} // namespace kernel
} // namespace nnfw

