/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <OperationsUtils.h>
#include <arm_compute/core/TensorShape.h>
#include <arm_compute/core/TensorInfo.h>

#include <cassert>

// TODO: fix include path in CMakeFiles
#include "../IO_accessor.h"
#include "../shape.h"
#include "../CLUniqueTensor.h"
#include "../FullyConnected.h"

namespace nnfw {
namespace kernel {
namespace acl {

void sync_scheduler() {
  arm_compute::CLScheduler::get().sync();
}

bool fullyConnectedFloat32(const float* inputData, const nnfw::rt::Shape& inputShape,
                           const float* weightsData, const nnfw::rt::Shape& weightsShape,
                           const float* biasData, const nnfw::rt::Shape& biasShape,
                           int32_t activation,
                           float* outputData, const nnfw::rt::Shape& outputShape) {
  return common::fullyConnectedFloat32<CLUniqueTensor, arm_compute::CLFullyConnectedLayer,
  arm_compute::CLActivationLayer>(inputData, inputShape,
      weightsData, weightsShape,
      biasData, biasShape,
      activation,
      outputData, outputShape,
      sync_scheduler);
}

} // namespace acl
} // namespace kernel
} // namespace nnfw
