/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __NNFW_KERNEL_ACL_SHAPE_H__
#define __NNFW_KERNEL_ACL_SHAPE_H__

#include <OperationsUtils.h>
#include <arm_compute/core/TensorShape.h>
#include <arm_compute/core/TensorInfo.h>
#include <arm_compute/runtime/IFunction.h>
#include <cassert>

namespace nnfw {
namespace rt {

// TODO remove from this source and use it from runtime
uint32_t getSizeOfDimension(const Shape& shape, uint32_t dimensionIdx);

} // namespace rt
} // namespace nnfw

namespace nnfw {
namespace kernel {
namespace acl {
namespace util {

arm_compute::TensorShape fromVectorNNShape(const nnfw::rt::Shape& shape);
arm_compute::TensorShape fromNNShape(const nnfw::rt::Shape& shape);

template<class TensorT, class ActT>
void insertFusedActivationLayer(TensorT& out, int activation,
    std::vector<std::shared_ptr<arm_compute::IFunction>>& fns) {
  auto relu_f = std::make_shared<ActT>();

  switch(activation) {
  case ANEURALNETWORKS_FUSED_NONE:
    // DO NOTHING
    return;

  case ANEURALNETWORKS_FUSED_RELU:
    {
      const arm_compute::ActivationLayerInfo relu_info(arm_compute::ActivationLayerInfo::ActivationFunction::RELU);

      // Do in-place update
      relu_f->configure(out.ptr(), nullptr, relu_info);
    }
    break;

  case ANEURALNETWORKS_FUSED_RELU1:
    {
      const arm_compute::ActivationLayerInfo relu_info(arm_compute::ActivationLayerInfo::ActivationFunction::BOUNDED_RELU, 1.f);

      // Do in-place update
      relu_f->configure(out.ptr(), nullptr, relu_info);
    }
    break;

  case ANEURALNETWORKS_FUSED_RELU6:
    {
      const arm_compute::ActivationLayerInfo relu_info(arm_compute::ActivationLayerInfo::ActivationFunction::BOUNDED_RELU, 6.f);

      // Do in-place update
      relu_f->configure(out.ptr(), nullptr, relu_info);
    }
    break;

  default:
    assert("Undefined activation type." && 0);
    break;
  }

  fns.emplace_back(relu_f);
}

} // namespace util
} // namespace acl
} // namespace kernel
} // namespace nnfw

#endif // __NNFW_KERNEL_ACL_SHAPE_H__
