/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __NNFW_KERNEL_SUPPORT_H_TEMPORARY__
#define __NNFW_KERNEL_SUPPORT_H_TEMPORARY__

// NOTE these are not decided yet but need to be moved out from Conv2D
//      to separate NEON implementation to it's folder
// TODO move to some folder where it should be

#include <cassert>

#include "util/feature/Shape.h"

#include <OperationsUtils.h>

namespace nnfw
{
namespace support
{
namespace nnapi
{
namespace feature
{

// TODO Extract this function as utility function
// NOTE It is not a good design to access nnfw::rt::Shape nnfw_support_nnapi lib
nnfw::util::feature::Shape asFeatureShape(const nnfw::rt::Shape& shape);

} // namespace feature
} // namespace nnapi
} // namespace support
} // namespace nnfw

#include <arm_compute/core/ITensor.h>

#include "util/feature/Reader.h"

namespace nnfw
{
namespace support
{
namespace acl
{
namespace feature
{

template<typename T> class Reader;

template<> class Reader<float> final : public nnfw::util::feature::Reader<float>
{
public:
  Reader(arm_compute::ITensor *tensor) : _tensor{tensor}
  {
    assert(tensor->info()->data_type() == arm_compute::DataType::F32);
  }

public:
  float at(uint32_t ch, uint32_t row, uint32_t col) const override
  {
    return *ptr_to_element(ch, row, col);
  }

private:
  float *ptr_to_element(uint32_t ch, uint32_t row, uint32_t col) const
  {
    // ARM Compute uses CHW ordering
    return reinterpret_cast<float *>(_tensor->ptr_to_element(arm_compute::Coordinates{col, row, ch}));
  }

private:
  arm_compute::ITensor *_tensor;
};

} // namespace feature
} // namespace acl
} // namespace support
} // namespace nnfw

#endif // __NNFW_KERNEL_SUPPORT_H_TEMPORARY__
