/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "support/tflite/TensorView.h"

#include <cassert>

namespace nnfw
{
namespace support
{
namespace tflite
{

TensorView<float>::TensorView(const nnfw::util::tensor::Shape &shape, float *base) : _shape{shape}, _base{base}
{
  // Set 'stride'
  _stride.init(_shape);
}

float TensorView<float>::at(const nnfw::util::tensor::Index &index) const
{
  const auto offset = _stride.offset(index);

  return *(_base + offset);
}

float &TensorView<float>::at(const nnfw::util::tensor::Index &index)
{
  const auto offset = _stride.offset(index);

  return *(_base + offset);
}

TensorView<float> TensorView<float>::make(::tflite::Interpreter &interp, int tensor_index)
{
  auto tensor_ptr = interp.tensor(tensor_index);

  // TODO Enable the following assets
  // assert(isFloatTensor(tensor_ptr));
  // assert(isFeatureTensor(tensor_ptr));

  // Set 'shape'
  nnfw::util::tensor::Shape shape(tensor_ptr->dims->size);

  for (uint32_t axis = 0; axis < shape.rank(); ++axis)
  {
    shape.dim(axis) = tensor_ptr->dims->data[axis];
  }

  return TensorView<float>(shape, interp.typed_tensor<float>(tensor_index));
}

} // namespace tflite
} // namespace support
} // namespace nnfw
